package main

import (
	"bytes"
	"crypto/tls"
	"encoding/json"
	"fmt"
	"github.com/gin-gonic/gin"
	"io"
	"log"
	"net/http"
	"net/http/httputil"
	"net/url"
	"os"
	"path/filepath"
	"regexp"
	"strconv"
)

// 加载服务器配置文件
func loadConfig() {
	// 获取可执行文件所在目录  ./config.json
	exePath, _ := os.Executable()
	exeDir := filepath.Dir(exePath)
	configFileTemp, err := os.Open(exeDir + "/config.json")
	if err != nil {
		panic("file \"./config.json\" not found")
	}
	//函数退出时关闭文件流
	defer func(configFile *os.File) {
		_ = configFile.Close()
	}(configFileTemp)
	//解析json
	decoder := json.NewDecoder(configFileTemp)
	err = decoder.Decode(&configFile)
	if err != nil {
		panic("config format err")
	}
}

// 初始化Gin引擎
func initGinEngine() {
	// 设置gin模式为发布模式
	gin.SetMode(gin.ReleaseMode)
	//关闭gin日志输出  自认为小项目没什么用
	gin.DefaultWriter = io.Discard
	// 创建Gin引擎
	copilotGinEngine = gin.New()
	// 设置信任的前置代理  用nginx反代需要  不写这个编译会有个警告看着难受
	if err := copilotGinEngine.SetTrustedProxies([]string{"127.0.0.1"}); err != nil {
		log.Fatal(err)
	}
}

// StartServer 启动服务器并监听ip和端口
func StartServer() {
	//监听地址是host+port
	listenAddress := configFile.Server.Host + ":" + strconv.Itoa(configFile.Server.Port)
	server := &http.Server{
		Addr: listenAddress,
		TLSConfig: &tls.Config{
			NextProtos: []string{"http/1.1", "http/1.2", "http/2"},
		},
		Handler: copilotGinEngine,
	}
	go func() {
		if configFile.Server.Port != 443 {
			err := copilotGinEngine.Run(listenAddress)
			log.Fatal(err)
		} else {
			err := server.ListenAndServeTLS(configFile.Server.CertPath, configFile.Server.KeyPath)
			log.Fatal(err)
		}
	}()
}

// 自定义请求失败返回的状态码和错误信息
func diyBadRequest(c *gin.Context, code int, errorMessage string) {
	c.JSON(code, gin.H{
		"message":           errorMessage,
		"documentation_url": "https://docs.github.com/rest",
	})
}

// registerProxyRoute 反向代理路由
func registerProxyRoute(r *gin.Engine, routePath, targetURL string) {
	target, err := url.Parse(targetURL)
	if err != nil {
		panic(err)
	}
	proxy := &httputil.ReverseProxy{
		Director: func(req *http.Request) {
			req.URL.Host = target.Host
			req.URL.Scheme = "http"
			req.Host = target.Host
		},
		ErrorHandler: func(w http.ResponseWriter, r *http.Request, err error) {
		},
	}
	r.Any(routePath, func(c *gin.Context) {
		proxy.Director = func(req *http.Request) {
			req.URL.Host = target.Host
			req.URL.Scheme = "https"
			req.Host = target.Host
		}
		if configFile.IsModMsg {
			modProxyResp(c, proxy)
		}
		proxy.ServeHTTP(c.Writer, c.Request)

		c.Writer.Header().Set("X-Forwarded-Proto", "http")
	})
}

// 修改代码提示内容
func modProxyResp(c *gin.Context, proxy *httputil.ReverseProxy) {
	// 在代理响应之前修改响应
	proxy.ModifyResponse = func(response *http.Response) error {
		responseBuffer := new(bytes.Buffer)
		_, readErr := responseBuffer.ReadFrom(response.Body)
		if readErr != nil {
			return readErr
		}
		responseData := responseBuffer.Bytes()
		// 定义正则表达式模式
		pattern := `(.*?)data:`
		// 编译正则表达式
		regex := regexp.MustCompile(pattern)
		// 查找匹配项
		match := regex.FindStringSubmatch(string(responseData))
		var replacedData = ""
		if len(match) >= 2 {
			replacedData = match[1]
		} else {
			fmt.Println("No match found.")
		}
		// 将字符串转换为[]rune类型的字符数组
		runes := []rune(configFile.DiyMsg)
		newStr := ""
		// 遍历字符数组，对每个字符进行处理
		for _, r := range runes {
			// 在字符前后添加内容，生成新的字符串
			newStr += fmt.Sprintf("data: {\"id\":\"cmpl-7xy1GLgssjHEubVrPyt534VRYVF0t\",\"model\":\"cushman-ml\",\"created\":1694526422,\"choices\":[{\"text\":\"%c\",\"index\":0,\"finish_reason\":null,\"logprobs\":null}]}\n", r)
		}
		newStr = replacedData + newStr + "data: [DONE]\n"
		fmt.Println(newStr)
		c.Data(response.StatusCode, response.Header.Get("Content-Type"), []byte(newStr))
		return nil
	}
}
